/*
 *  LED_Remote_Color_Picker_Receiver
 *  Receives color setting commands and shows them
 *  
 *  On the shoulders of giants - uses PainlessMesh and FastLED
 *  
 *  2021 Carl F Sutter
 *  
 */

// LED Setup - must come before the painlessMesh so CRGB can be used as a return type
#include <FastLED.h>
// How many leds in your strip?
#define NUM_LEDS 338
// For led chips like Neopixels, which have a data line, ground, and power, you just
// need to define DATA_PIN.  Note that the GPIO and Data pins are laballed separately on ESP8266 boards
#define DATA_PIN D6
// Define the array of leds
CRGB leds[NUM_LEDS];


// Global App Vars
// The timer is used to refresh the pixels in the lop in case the power goes out
unsigned long ul_PreviousMillis = 0UL;   // using millis rollver code from http://playground.arduino.cc/Code/TimingRollover
unsigned long ul_Interval = 1000UL;      // refresh once a second
uint32_t current_color = 0xFF0000;       // the current color


// Mesh Setup
#include "painlessMesh.h"
#define   MESH_PREFIX     "ColorPicker"
#define   MESH_PASSWORD   "foobarbaz"       // needs to be at leats 8 characters!
#define   MESH_PORT       5555
painlessMesh  mesh;

// this gets called when the sender sends a new color command
void receivedCallback(uint32_t from, String &msg) {
  Serial.print("Received color: ");
  Serial.println(msg);
  current_color = (uint32_t) msg.toInt();

  // set the LEDS to the new color
    setLEDs();
   
  //ul_PreviousMillis = 0UL;  // another approach, but the message delay seems to add latency
} // receivedCallback


// called on a new connection - ignore since we will used changedConnection
void newConnectionCallback(uint32_t nodeId) {
    Serial.printf("--> startHere: New Connection, nodeId = %u\n", nodeId);
} // newConnectionCallback


// this gets called when a node is added or removed from the mesh, so set the controller to the node with the lowest chip id
void changedConnectionCallback() {
  Serial.printf("Changed connections %s\n",mesh.subConnectionJson().c_str());
} // changedConnectionCallback


// setup
void setup() {
  Serial.begin(9600);
  
  FastLED.addLeds<WS2812B, DATA_PIN, RGB>(leds, NUM_LEDS);
  //FastLED.setBrightness(60);
  
  //mesh.setDebugMsgTypes( ERROR | MESH_STATUS | CONNECTION | SYNC | COMMUNICATION | GENERAL | MSG_TYPES | REMOTE ); // all types on
  mesh.setDebugMsgTypes( ERROR | STARTUP );  // set before init() so that you can see startup messages
  mesh.init( MESH_PREFIX, MESH_PASSWORD, MESH_PORT );
  mesh.onReceive(&receivedCallback);
  mesh.onNewConnection(&newConnectionCallback);
  mesh.onChangedConnections(&changedConnectionCallback);
  Serial.print("My Node ID: ");
  Serial.println(mesh.getNodeId());

  ul_PreviousMillis = millis();
} // setup


// loop
void loop() {
  mesh.update();

  // if the alloted time (~1sec) has passed, re-paint the LEDS in case of power outrage etc.
  unsigned long ul_CurrentMillis = millis();  
  if (ul_CurrentMillis - ul_PreviousMillis > ul_Interval) {
    setLEDs();
    ul_PreviousMillis = millis();
  }
} // loop


// draw the LEDs with the current color
void setLEDs(void) {
  // coloring in every fourth LED for a patio light effect
  for(uint16_t i=0; i<NUM_LEDS; i++) { 
    if (i%4 == 0) leds[i] = current_color;
  }
  FastLED.show();
} // setLEDs
